﻿using System.Collections.Generic;
using System.Linq;
using SmartApp.IotDevice;
using SmartApp.IotDevice.Repositories;
using SmartApp.Services;

namespace SmartApp.Factory
{
    public sealed class DependencyFactory
    {
        #region members

        private static List<object> _registeredInstances = null;

        #endregion
        
        public static T GetInstance<T>()
        {
            // check if initialized
            if (_registeredInstances == null)
            {
                _registeredInstances = new List<object>(); // init for list

                var heatingCentral = new HeatingCentral();
                var heatingBath = new HeatingBath();
                var heatingOffice = new HeatingOffice();
                var parameterRepositoryFile = new ParameterRepositoryFile();
                var lightsBath = new LightsBath();
                var lightsOffice = new LightsOffice();
                var lightsEntry = new LightsEntry();
                var heatingService = new HeatingService(heatingCentral, heatingBath, heatingOffice,
                    parameterRepositoryFile);

                RegisterInstance(heatingBath);
                RegisterInstance(heatingOffice);
                RegisterInstance(heatingCentral);
                RegisterInstance(heatingBath);
                RegisterInstance(heatingOffice);
                RegisterInstance(lightsBath);
                RegisterInstance(lightsOffice);
                RegisterInstance(lightsEntry);
                RegisterInstance(parameterRepositoryFile);
                RegisterInstance(heatingService);
                RegisterInstance(new HeatingManagementService(heatingBath, heatingOffice, heatingCentral, heatingService));
                RegisterInstance(new LightBathService(lightsBath));
                RegisterInstance(new LightOfficeService(lightsOffice));
                RegisterInstance(new LightEntryService(lightsEntry));
                RegisterInstance(new SwitchControllerService(lightsBath, lightsOffice, lightsEntry));
            }

            T ret = default(T);
            // todo: if you have 2 classes which implement same interface to that is a problem
            foreach (T registeredInstance in _registeredInstances.OfType<T>())
            {
                ret = registeredInstance;
                break;
            }

            return ret;
        }

        private static void RegisterInstance<T>(T instance)
        {
            _registeredInstances.Add(instance);
        }

    }
}
