﻿using Windows.ApplicationModel.Background;
using SmartApp.Contracts.Services;
using SmartApp.Factory;

// The Background Application template is documented at http://go.microsoft.com/fwlink/?LinkID=533884&clcid=0x409

namespace SmartApp.Host.HeatingController
{
    public sealed class StartupTask : IBackgroundTask
    {
        private BackgroundTaskDeferral _deferral = null; // needed - something to do with GC cleaning problems

        private IHeatingRoomService _heatingController = null; // logic to control heating I/Os

        public void Run(IBackgroundTaskInstance taskInstance)
        {
            _deferral = taskInstance.GetDeferral(); // -> don´t release deferral, otherwise app will stop
            taskInstance.Canceled += BackgroundTaskCanceled;

            if (_heatingController != null)
            {
                // clean up existing stuff
                _heatingController.StopControlHeating();

                while (!_heatingController.IsStopped)
                {
                    // nothing to do
                }
            }
            else
            {
                _heatingController = DependencyFactory.GetInstance<IHeatingRoomService>();
            }

            _heatingController.StartControlHeating();
        }

        private void BackgroundTaskCanceled(IBackgroundTaskInstance sender, BackgroundTaskCancellationReason reason)
        {
            if (_heatingController != null)
            {
                _heatingController.StopControlHeating();

                while (!_heatingController.IsStopped)
                {
                    // nothing to do
                }
            }

            _deferral?.Complete();
        }
    }
}
