﻿using Windows.ApplicationModel.Background;
using SmartApp.Contracts.Services;
using SmartApp.Factory;

// The Background Application template is documented at http://go.microsoft.com/fwlink/?LinkID=533884&clcid=0x409

namespace SmartApp.Host.SwitchController
{
    public sealed class StartupTask : IBackgroundTask
    {
        private BackgroundTaskDeferral _deferral = null; // needed - something to do with GC cleaning problems

        private ISwitchService _switchController = null; // logic to control switch I/Os

        public void Run(IBackgroundTaskInstance taskInstance)
        {
            _deferral = taskInstance.GetDeferral(); // -> don´t release deferral, otherwise app will stop
            taskInstance.Canceled += BackgroundTaskCanceled;

            if (_switchController != null)
            {
                // clean up existing stuff
                _switchController.StopControlSwitches();

                while (!_switchController.IsStopped)
                {
                    // nothing to do
                }
            }
            else
            {
                _switchController = DependencyFactory.GetInstance<ISwitchService>();
            }

            _switchController.StartControlSwitches();
        }

        private void BackgroundTaskCanceled(IBackgroundTaskInstance sender, BackgroundTaskCancellationReason reason)
        {
            if (_switchController != null)
            {
                _switchController.StopControlSwitches();

                while (!_switchController.IsStopped)
                {
                    // nothing to do
                }
            }

            _deferral?.Complete();
        }
    }
}
