﻿using Windows.ApplicationModel.Background;
using Restup.Webserver.Http;
using Restup.Webserver.Rest;
using SmartApp.WebPortal;
using Restup.Webserver.File;

// The Background Application template is documented at http://go.microsoft.com/fwlink/?LinkID=533884&clcid=0x409

namespace SmartApp.Host.Webserver
{
    public sealed class StartupTask : IBackgroundTask
    {
        private HttpServer _httpServer = null;

        private BackgroundTaskDeferral _deferral = null; // needed - something to do with GC cleaning problems

        public async void Run(IBackgroundTaskInstance taskInstance)
        {
            _deferral = taskInstance.GetDeferral(); // -> don´t release deferral, otherwise app will stop
            taskInstance.Canceled += BackgroundTaskCanceled;

            // register handlers for webserver
            var restRouteHandler = new RestRouteHandler();
            restRouteHandler.RegisterController<HeatingManagementController>();
            restRouteHandler.RegisterController<LightBathController>();
            restRouteHandler.RegisterController<LightOfficeController>();
            restRouteHandler.RegisterController<LightEntryController>();

            var configuration = new HttpServerConfiguration()
            .ListenOnPort(80)
            .RegisterRoute("api/v1", restRouteHandler)
            .RegisterRoute(new StaticFileRouteHandler(@"SmartApp.WebPortal\Web\"))
            .EnableCors(); // allow cors requests on all origins 

            // webserver
            _httpServer = new HttpServer(configuration);
            // start webserver
            await _httpServer.StartServerAsync();
        }

        private void BackgroundTaskCanceled(IBackgroundTaskInstance sender, BackgroundTaskCancellationReason reason)
        {
            // stop webserver
            _httpServer?.StopServer();

            _deferral?.Complete();
        }
    }
}
