﻿using Windows.Devices.Gpio;
using SmartApp.Contracts.IO;
using System;
using SmartApp.IotDevice.Modules;

namespace SmartApp.IotDevice
{
    public sealed class LightsBath : ILightsBath
    {
        #region members

        private readonly IDigitalOutputModule _digitalOutputModule0;
        
        private readonly IDigitalInputModule _digitalInputModule0;

        #endregion

        #region constructor
        
        public LightsBath()
        {
            _digitalOutputModule0 = new DigitalOutputModule();
            _digitalOutputModule0.InitializeDevice(I2CAdresses.DigitalOutputModule0);

            _digitalInputModule0 = new DigitalInputModule();
            _digitalInputModule0.InitializeDevice(I2CAdresses.DigitalInputModule0);
        }

        #endregion

        public bool SwitchEntry => (_digitalInputModule0.GetData() & 0x08) > 0; // 2 switches on one input
        public bool SwitchShower => (_digitalInputModule0.GetData() & 0x08) > 0; // 2 switches on one input

        public bool BalkLight
        {
            get
            {
                // 0 -> b3
                return (_digitalOutputModule0.ReadData() & 0x08) > 0;
            }
            set
            {
                var d = _digitalOutputModule0.ReadData();
                d = value ? Convert.ToByte(d | 0x08) : Convert.ToByte(d & 0xF7);

                _digitalOutputModule0.SetData(d);
            }
        }

        public bool ShowerLight
        {
            get
            {
                // 0 -> b4
                return (_digitalOutputModule0.ReadData() & 0x10) > 0;
            }
            set
            {
                var d = _digitalOutputModule0.ReadData();
                d = value ? Convert.ToByte(d | 0x10) : Convert.ToByte(d & 0xEF);

                _digitalOutputModule0.SetData(d);
            }
        }
    }
}
