﻿using System;
using SmartApp.Contracts.IO;
using SmartApp.IotDevice.Modules;

namespace SmartApp.IotDevice
{
    public sealed class LightsOffice : ILightsOffice
    {
        #region members
        
        private readonly IDigitalOutputModule _digitalOutputModule0;

        private readonly IDigitalOutputModule _digitalOutputModule1;

        private readonly IDigitalInputModule _digitalInputModule0;

        #endregion

        #region constructor

        public LightsOffice()
        {
            _digitalOutputModule0 = new DigitalOutputModule();
            _digitalOutputModule0.InitializeDevice(I2CAdresses.DigitalOutputModule0);

            _digitalOutputModule1 = new DigitalOutputModule();
            _digitalOutputModule1.InitializeDevice(I2CAdresses.DigitalOutputModule1);

            _digitalInputModule0 = new DigitalInputModule();
            _digitalInputModule0.InitializeDevice(I2CAdresses.DigitalInputModule0);
        }

        #endregion

        public bool SwitchMediaboard1 => (_digitalInputModule0.GetData() & 0x02) > 0;
        public bool SwitchMediaboard2 => (_digitalInputModule0.GetData() & 0x04) > 0;
        public bool BalkLight1
        {
            get
            {
                // 0 -> b5
                return (_digitalOutputModule0.ReadData() & 0x20) > 0;
            }
            set
            {
                var d = _digitalOutputModule0.ReadData();
                d = value ? Convert.ToByte(d | 0x20) : Convert.ToByte(d & 0xDF);

                _digitalOutputModule0.SetData(d);
            }
        }
      
        public bool BalkLight2
        {
            get
            {
                // 0 -> b6
                return (_digitalOutputModule0.ReadData() & 0x40) > 0;
            }
            set
            {
                var d = _digitalOutputModule0.ReadData();
                d = value ? Convert.ToByte(d | 0x40) : Convert.ToByte(d & 0xBF);

                _digitalOutputModule0.SetData(d);
            }
        }

        public bool BalkLight5And6
        {
            get
            {
                // 0 -> b7
                return (_digitalOutputModule0.ReadData() & 0x80) > 0;
            }
            set
            {
                var d = _digitalOutputModule0.ReadData();
                d = value ? Convert.ToByte(d | 0x80) : Convert.ToByte(d & 0x7F);

                _digitalOutputModule0.SetData(d);
            }
        }

        public bool BalkLight7And8
        {
            get
            {
                // 1 -> b0
                return (_digitalOutputModule1.ReadData() & 0x01) > 0;
            }
            set
            {
                var d = _digitalOutputModule1.ReadData();
                d = value ? Convert.ToByte(d | 0x01) : Convert.ToByte(d & 0xFE);

                _digitalOutputModule1.SetData(d);
            }
        }

        public bool BalkLight9
        {
            get
            {
                // 1 -> b1
                return (_digitalOutputModule1.ReadData() & 0x02) > 0;
            }
            set
            {
                var d = _digitalOutputModule1.ReadData();
                d = value ? Convert.ToByte(d | 0x02) : Convert.ToByte(d & 0xFD);

                _digitalOutputModule1.SetData(d);
            }
        }

        public bool BalkLight10
        {
            get
            {
                // 1 -> b2
                return (_digitalOutputModule1.ReadData() & 0x04) > 0;
            }
            set
            {
                var d = _digitalOutputModule1.ReadData();
                d = value ? Convert.ToByte(d | 0x04) : Convert.ToByte(d & 0xFB);

                _digitalOutputModule1.SetData(d);
            }
        }

        public bool BalkLightUpper
        {
            get
            {
                // 1 -> b3
                return (_digitalOutputModule1.ReadData() & 0x08) > 0;
            }
            set
            {
                var d = _digitalOutputModule1.ReadData();
                d = value ? Convert.ToByte(d | 0x08) : Convert.ToByte(d & 0xF7);

                _digitalOutputModule1.SetData(d);
            }
        }

        public bool MediaBoardLight
        {
            get
            {
                // 1 -> b5
                return (_digitalOutputModule1.ReadData() & 0x20) > 0;
            }
            set
            {
                var d = _digitalOutputModule1.ReadData();
                d = value ? Convert.ToByte(d | 0x20) : Convert.ToByte(d & 0xDF);

                _digitalOutputModule1.SetData(d);
            }
        }

        public bool CupboardLight
        {
            get
            {
                // 1 -> b4
                return (_digitalOutputModule1.ReadData() & 0x10) > 0;
            }
            set
            {
                var d = _digitalOutputModule1.ReadData();
                d = value ? Convert.ToByte(d | 0x10) : Convert.ToByte(d & 0xEF);

                _digitalOutputModule1.SetData(d);
            }
        }
    }
}
