﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using Windows.Devices.Enumeration;
using Windows.Devices.I2c;

namespace SmartApp.IotDevice.Modules
{
    /// <summary>
    /// I only provided a subset of original functionality (e.g. no calibration).
    /// A full sample in c you can find here: https://github.com/BoschSensortec/BMP280_driver/blob/master/bmp280.c
    /// </summary>
    public class BoschBmp280 : IBoschBmp280
    {
        private I2cDevice _sensorDevice = null;
        private byte _deviceId = 0;
        
        public byte GetDeviceId()
        {
            return 1;
        }

        public bool InitializeDevice(byte deviceId)
        {
            _deviceId = deviceId;
            var t = this.InitializeInternal(deviceId);
            return t.Result;
        }

        private async Task<bool> InitializeInternal(byte deviceId)
        {
            var i2CDeviceSelector = I2cDevice.GetDeviceSelector();
            IReadOnlyList<DeviceInformation> devices = await DeviceInformation.FindAllAsync(i2CDeviceSelector);

            var HTU21D_settings = new I2cConnectionSettings(deviceId);
            _sensorDevice = await I2cDevice.FromIdAsync(devices[0].Id, HTU21D_settings);

            return true;
        }

        public BoschBmp280Data ReadData()
        {
            // possible something like that here:
            // Read data from I2C. 
            var command = new byte[1];
            var humidityData = new byte[2];
            var temperatureData = new byte[2];
            

            // Read humidity. 
            command[0] = 0xE5;

            // If this next line crashes with a NullReferenceException, then 
            // there was a sharing violation on the device. (See StartScenarioAsync above.) 
            // 
            // If this next line crashes for some other reason, then there was 
            // an error accessing the device. 
            _sensorDevice.WriteRead(command, humidityData);

              // Read temperature. 
            command[0] = 0xE3;
            // If this next line crashes, then there was an error accessing the sensor. 
            _sensorDevice.WriteRead(command, temperatureData);

       // Calculate and report the humidity. 
       var rawHumidityReading = humidityData[0] << 8 | humidityData[1];
                   var humidityRatio = rawHumidityReading / (float)65536;
                   double humidity = -6 + (125 * humidityRatio);
       

            
            // Calculate and report the temperature. 
            var rawTempReading = temperatureData[0] << 8 | temperatureData[1];
                       var tempRatio = rawTempReading / (float)65536;
                       double temperature = (-46.85 + (175.72 * tempRatio)) * 9 / 5 + 32;
            
            return new BoschBmp280Data()
            {
                AmbientPressure = 1.0,
                DeviceId = 1,
                Humidity = 20,
                Temperature = 21
            };
        }
    }
}
