﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Threading.Tasks;
using System.Xml;
using System.Xml.Serialization;
using Windows.ApplicationModel;
using Windows.Data.Json;
using Windows.Storage;
using SmartApp.Contracts.Models;
using SmartApp.Contracts.Repositories;

namespace SmartApp.IotDevice.Repositories
{
    /// <summary>
    /// Provides read and write operations to manage parameters from file system.
    /// </summary>
    public class ParameterRepositoryFile : IParameterRepository
    {
        #region members

        private static string _pathToParameterHeatingFile;

        #endregion

        #region constructor

        public ParameterRepositoryFile()
        {
            _pathToParameterHeatingFile = Path.Combine(Windows.Storage.ApplicationData.Current.LocalFolder.Path, "parameterHeatings.xml");
        }

        #endregion

        #region IParameterRepository

        public void DeleteParameterHeating(ParameterHeating parameterHeating)
        {
            var parameters = this.GetFileContent().Result;

            var index = 0;
            for (var i = 0; i < parameters.Count -1; i++)
            {
                if (parameters[i].Id == parameterHeating.Id)
                {
                    index = i;
                }
            }
            
            parameters.RemoveAt(index);

            this.SaveFileContent(parameters);
        }

        public async Task<List<ParameterHeating>> GetParameterHeatings()
        {
            if (!File.Exists(_pathToParameterHeatingFile))
            {
                // create default parameters if nothing is existing
                var defaultParams = new List<ParameterHeating>();
                defaultParams.Add(new ParameterHeating()
                {
                    Id = "1",
                    StartHour = 5,
                    StoppHour = 23,
                    TargetTemperature = 21.5,
                    Variance = 2.0
                });

                this.SaveFileContent(defaultParams);
            }

            return await this.GetFileContent();
        }

        public void SaveParameterHeatings(List<ParameterHeating> parameterHeatings)
        {
            this.SaveFileContent(parameterHeatings);
        }

        #endregion

        #region file handling

        /// <summary>
        /// Save data to file system.
        /// </summary>
        /// <param name="data"></param>
        private async void SaveFileContent(List<ParameterHeating> data)
        {
            XmlSerializer serializer = new XmlSerializer(typeof(List<ParameterHeating>));
             
            var f = await StorageFile.GetFileFromPathAsync(_pathToParameterHeatingFile);
            using (var stream = await f.OpenStreamForWriteAsync())
            {
                XmlReader reader = XmlReader.Create(stream);
                serializer.Serialize(stream, data);
            }
        }

        /// <summary>
        /// Get content of file as typed data.
        /// </summary>
        /// <returns></returns>
        private async Task<List<ParameterHeating>> GetFileContent()
        {
            List<ParameterHeating> content = null;
            XmlSerializer serializer = new XmlSerializer(typeof(List<ParameterHeating>));

            var f = await StorageFile.GetFileFromPathAsync(_pathToParameterHeatingFile);
            using (var stream = await f.OpenStreamForReadAsync())
            {
                XmlReader reader = XmlReader.Create(stream);
                content = (List<ParameterHeating>)serializer.Deserialize(reader);
            }
            
            return content;
        }

        #endregion
    }
}
