﻿using SmartApp.Contracts.Services;
using System.Collections.Generic;
using System.Linq;
using SmartApp.Contracts.IO;
using SmartApp.Contracts.Models;

namespace SmartApp.Services
{
    public sealed class HeatingManagementService : IHeatingManagementService
    {
        #region members

        private readonly IHeatingRoom _bath = null;

        private readonly IHeatingRoom _office = null;

        private readonly IHeatingCentral _central = null;

        private readonly IHeatingRoomService _heating = null;

        #endregion

        #region constructor

        public HeatingManagementService(IHeatingRoom bath, IHeatingRoom office, IHeatingCentral central, IHeatingRoomService heatingRoomService)
        {
            _bath = bath;
            _office = office;
            _central = central;
            _heating = heatingRoomService;
        }

        #endregion

        #region IHeatingManagementService
        
        public void SetHeatingParameter(ParameterHeating parameter)
        {
            var par = _heating.GetParameterHeating();

            // find out, if id is still extisting -> Update otherwise ADD
            // not parameter does exist, so add a new one
            if (par == null)
            {
                par = new List<ParameterHeating>();
                par.Add(parameter);
            }
            else
            {
                // check if update or add is required
                var p = (from ps in par
                         where ps.Id == parameter.Id
                         select ps).SingleOrDefault();

                if (p == null)
                {
                    // add
                    par.Add(parameter);
                }
                else
                {
                    // update
                    par.Remove(p);
                    par.Add(parameter);
                }
            }

            _heating.SaveParameterHeating(par);
        }

        public void DeleteHeatingParameter(string parameterHeadingId)
        {
            var par = _heating.GetParameterHeating();

            if (par != null)
            {
                foreach (var parameterHeating in par.Where(parameterHeating => parameterHeating.Id == parameterHeadingId))
                {
                    par.Remove(parameterHeating);
                }
            }

            _heating.SaveParameterHeating(par);
        }

        public List<ParameterHeating> GetHeatingParameter()
        {
           return _heating.GetParameterHeating();
        }

       
        public IHeatingRoom GetHeatingStatesBath()
        {
            return _bath;
        }

        public IHeatingRoom GetHeatingStatesOffice()
        {
            return _office;
        }

        public IHeatingCentral GetHeatingStatesCentral()
        {
            return _central;
        }

        public void SetCentralPump(bool state)
        {
            _central.CentralPump = state;
        }

        public void SetHeatingCircuitValveBath(bool state)
        {
            _bath.CircuitValve = state;
        }

        public void SetHeatingCircuitValveOffice(bool state)
        {
            _office.CircuitValve = state;
        }

        public void SetServiceMode(bool serviceMode)
        {
            _heating.ServiceMode = serviceMode;
        }

        public bool GetServiceMode()
        {
            return _heating.ServiceMode;
        }

        #endregion
    }
}
