﻿using System;
using System.Collections.Generic;
using System.Linq;
using SmartApp.Contracts.IO;
using SmartApp.Contracts.Models;
using SmartApp.Contracts.Repositories;

namespace SmartApp.Services
{
    public sealed class HeatingService : Contracts.Services.IHeatingRoomService
    {
        #region members
        
        private bool _stoppController = false;

        private readonly IHeatingCentral _heatingCentral = null;

        private readonly IHeatingRoom _heatingBath = null;

        private readonly IHeatingRoom _heatingOffice = null;

        private readonly IParameterRepository _parameterRepository = null;

        #endregion

        #region constructor

        public HeatingService(IHeatingCentral heatingCentral, IHeatingRoom heatingBath, IHeatingRoom heatingOffice, IParameterRepository parameterRepository)
        {
            _heatingCentral = heatingCentral;
            _heatingBath = heatingBath;
            _heatingOffice = heatingOffice;
            _parameterRepository = parameterRepository;
        }


        #endregion

        #region IHeatingRoomService

        public bool IsStopped { get; private set; }

        public bool ServiceMode { get; set; }

        public List<ParameterHeating> GetParameterHeating()
        {
            return this._parameterRepository.GetParameterHeatings().Result;
        }

        public void SaveParameterHeating(List<ParameterHeating> parameterHeating)
        {
            this._parameterRepository.SaveParameterHeatings(parameterHeating);
        }
        
        public void StartControlHeating()
        {
            IsStopped = false;
            _stoppController = false;
            
            ControlHeating();
        }

        public void StopControlHeating()
        {
            _stoppController = true;
        }

        #endregion
        
        #region private operations to control heating stuff

         private void ControlHeating()
        {
            while (!_stoppController)
            {
                if (!ServiceMode)
                {
                    ControlRooms();

                    ControlCentral();
                }
            }

            IsStopped = true;
        }

        private void ControlRooms()
        {
            ParameterHeating usedParameter = null;
            var parameters = _parameterRepository.GetParameterHeatings().Result;

            // find out what for parameter we have to use (current time have to be in timeslot between start and stop)
            foreach (var parameterHeating in parameters.Where(parameterHeating => DateTime.Now.Hour > parameterHeating.StartHour & DateTime.Now.Hour > parameterHeating.StoppHour))
            {
                usedParameter = parameterHeating;
            }

            if (usedParameter == null)
            {
                _heatingBath.CircuitValve = false;
                _heatingOffice.CircuitValve = false;
            }
            else
            {
                // ****** Bath **********
                // check if temperatur is to hot --> turn off
                if ((usedParameter.TargetTemperature + usedParameter.Variance) > _heatingBath.Temperature)
                {
                    _heatingBath.CircuitValve = false;
                }

                // check if temperatur is to cold --> turn on
                if ((usedParameter.TargetTemperature - usedParameter.Variance) < _heatingBath.Temperature)
                {
                    _heatingBath.CircuitValve = true;
                }

                // ****** Office **********
                // check if temperatur is to hot --> turn off
                if ((usedParameter.TargetTemperature + usedParameter.Variance) > _heatingOffice.Temperature)
                {
                    _heatingOffice.CircuitValve = false;
                }

                // check if temperatur is to cold --> turn on
                if ((usedParameter.TargetTemperature - usedParameter.Variance) < _heatingOffice.Temperature)
                {
                    _heatingOffice.CircuitValve = true;
                }
            }
        }
        
        private void ControlCentral()
        {
            _heatingCentral.CentralPump = _heatingBath.CircuitValve | _heatingOffice.CircuitValve;
        }

        #endregion
    }
}
