﻿using SmartApp.Contracts.IO;

namespace SmartApp.Services
{
    public sealed class SwitchControllerService : Contracts.Services.ISwitchService
    {
        #region members
        
        private bool _stoppController = false;

        private readonly ILightsBath _lightsBath = null;

        private readonly ILightsOffice _lightsOffice = null;

        private readonly ILightsEntry _lightsEntry = null;
        
        #endregion

        #region constructor

        public SwitchControllerService(ILightsBath lightsBath, ILightsOffice lightsOffice, ILightsEntry lightsEntry)
        {
            _lightsBath = lightsBath;
            _lightsOffice = lightsOffice;
            _lightsEntry = lightsEntry;
        }

        #endregion

        public bool IsStopped { get; private set; }

        public void StartControlSwitches()
        {
            IsStopped = false;
            _stoppController = false;

            ControlSwitches();
        }

        public void StopControlSwitches()
        {
            _stoppController = true;
        }

        private void ControlSwitches()
        {
            int switchMediaboard2Cnt = 0;
            bool forward = true;
           
            var switchEntryChecker = new SwitchLogic(this._lightsEntry.SwitchEntry);
            var switchMediaboard1Checker = new SwitchLogic(this._lightsOffice.SwitchMediaboard1);
            var switchMediaboard2Checker = new SwitchLogic(this._lightsOffice.SwitchMediaboard2);
            var switchBathRoomEntryChecker = new SwitchLogic(this._lightsBath.SwitchEntry);
            var switchBathRoomShowerChecker = new SwitchLogic(this._lightsBath.SwitchShower);


            while (!_stoppController)
            {
                // entry lights
                if (switchEntryChecker.StateChanged(this._lightsEntry.SwitchEntry) | switchMediaboard1Checker.StateChanged(this._lightsOffice.SwitchMediaboard1))
                {
                    if (this._lightsEntry.SwitchEntry | this._lightsOffice.SwitchMediaboard1)
                    {
                        this._lightsEntry.EntryMainLight = !this._lightsEntry.EntryMainLight;
                    }
                }

                // office lights
                if (switchMediaboard2Checker.StateChanged(this._lightsOffice.SwitchMediaboard2))
                {
                    if (this._lightsOffice.SwitchMediaboard2)
                    {
                        switchMediaboard2Cnt ++;

                        if ((switchMediaboard2Cnt > 8) & forward)
                        {
                            forward = false;
                            switchMediaboard2Cnt = 2; // skip reset
                        }

                        if ((switchMediaboard2Cnt > 8) & !forward)
                        {
                            forward = true;
                            switchMediaboard2Cnt = 1; // reset
                        }

                        switch (switchMediaboard2Cnt)
                        {
                            case 1:
                                // known state needed (otherwise no complete false state possible)
                                this._lightsOffice.BalkLight1 = false;
                                this._lightsOffice.BalkLight2 = false;
                                this._lightsOffice.BalkLight5And6 = false;
                                this._lightsOffice.BalkLight7And8 = false;
                                this._lightsOffice.BalkLight9 = false;
                                this._lightsOffice.BalkLight10 = false;
                                this._lightsOffice.CupboardLight = false;
                                this._lightsOffice.MediaBoardLight = false;
                                this._lightsOffice.BalkLightUpper = false;
                                break;

                            case 2:
                                this._lightsOffice.BalkLight1 = !this._lightsOffice.BalkLight1;
                                break;

                            case 3:
                                this._lightsOffice.BalkLight2 = !this._lightsOffice.BalkLight2;
                                break;

                            case 4:
                                this._lightsOffice.BalkLight5And6 = !this._lightsOffice.BalkLight5And6;
                                break;

                            case 5:
                                this._lightsOffice.BalkLight7And8 = !this._lightsOffice.BalkLight7And8;
                                break;

                            case 6:
                                this._lightsOffice.BalkLight9 = !this._lightsOffice.BalkLight9;
                                break;

                            case 7:
                                this._lightsOffice.BalkLight10 = !this._lightsOffice.BalkLight10;
                                break;

                            case 8:
                                this._lightsOffice.CupboardLight = !this._lightsOffice.CupboardLight;
                                this._lightsOffice.MediaBoardLight = !this._lightsOffice.MediaBoardLight;
                                this._lightsOffice.BalkLightUpper = !this._lightsOffice.BalkLightUpper;
                                break;
                        }
                    }
                }

                if (switchBathRoomEntryChecker.StateChanged(this._lightsBath.SwitchEntry) | switchBathRoomShowerChecker.StateChanged(this._lightsBath.SwitchShower))
                {
                    if (this._lightsBath.SwitchEntry | this._lightsBath.SwitchShower)
                    {
                        this._lightsBath.BalkLight = !this._lightsBath.BalkLight;
                        this._lightsBath.ShowerLight = !this._lightsBath.ShowerLight;
                    }
                }
            }

            IsStopped = true;
        }

       
    }
}
