﻿using SmartApp.Contracts.Services;
using SmartApp.Contracts.Models;
using Restup.Webserver.Attributes;
using Restup.Webserver.Models.Contracts;
using Restup.Webserver.Models.Schemas;

namespace SmartApp.WebPortal
{
    [RestController(InstanceCreationType.Singleton)]
    public sealed class HeatingManagementController
    {
        #region members

        private static IHeatingManagementService _heatingManagementService = null;

        #endregion

        #region constructor

        public HeatingManagementController()
        {
            _heatingManagementService = Factory.DependencyFactory.GetInstance<IHeatingManagementService>();
        }

        #endregion

        #region HeatingManagementService methods

        [UriFormat("/heatingmangement/heatingparameter")]
        public IPutResponse SetHeatingParameter([FromContent] ParameterHeating parameter)
        {
            _heatingManagementService.SetHeatingParameter(parameter);
            return new PutResponse(PutResponse.ResponseStatus.OK);
        }

        [UriFormat("/heatingmangement/heatingparameter/{parameterHeadingId}")]
        public IDeleteResponse DeleteHeatingParameter(string parameterHeadingId)
        {
            _heatingManagementService.DeleteHeatingParameter(parameterHeadingId);
            return new DeleteResponse(DeleteResponse.ResponseStatus.OK);
        }

        [UriFormat("/heatingmangement/heatingparameter")]
        public IGetResponse GetHeatingParameter()
        {
            var p = _heatingManagementService.GetHeatingParameter();

            return new GetResponse(GetResponse.ResponseStatus.OK, p);
        }

        [UriFormat("/heatingmangement/heatingstatesbath")]
        public IGetResponse GetHeatingStatesBath()
        {
            var states = _heatingManagementService.GetHeatingStatesBath();
            return new GetResponse(GetResponse.ResponseStatus.OK, states);
        }

        [UriFormat("/heatingmangement/heatingstatesoffice")]
        public IGetResponse GetHeatingStatesOffice()
        {
            var states = _heatingManagementService.GetHeatingStatesOffice();
            return new GetResponse(GetResponse.ResponseStatus.OK, states);
        }

        [UriFormat("/heatingmangement/heatingstatescentral")]
        public IGetResponse GetHeatingStatesCentral()
        {
            var states = _heatingManagementService.GetHeatingStatesCentral();
            return new GetResponse(GetResponse.ResponseStatus.OK, states);
        }

        [UriFormat("/heatingmangement/centralpump/{state}")]
        public IPutResponse SetCentralPump(bool state)
        {
            _heatingManagementService.SetCentralPump(state);
            return new PutResponse(PutResponse.ResponseStatus.OK);
        }

        [UriFormat("/heatingmangement/heatingcircuitvalvebath/{state}")]
        public IPutResponse SetHeatingCircuitValveBath(bool state)
        {
            _heatingManagementService.SetHeatingCircuitValveBath(state);
            return new PutResponse(PutResponse.ResponseStatus.OK);
        }

        [UriFormat("/heatingmangement/heatingcircuitvalveoffice/{state}")]
        public IPutResponse SetHeatingCircuitValveOffice(bool state)
        {
            _heatingManagementService.SetHeatingCircuitValveOffice(state);
            return new PutResponse(PutResponse.ResponseStatus.OK);
        }

        [UriFormat("/heatingmangement/servicemode/{servicemode}")]
        public IPutResponse SetServiceMode(bool serviceMode)
        {
            _heatingManagementService.SetServiceMode(serviceMode);
            return new PutResponse(PutResponse.ResponseStatus.OK);
        }

        [UriFormat("/heatingmangement/servicemode")]
        public IGetResponse GetServiceMode()
        {
            _heatingManagementService = Factory.DependencyFactory.GetInstance<IHeatingManagementService>();
            var mode = _heatingManagementService.GetServiceMode();
            return new GetResponse(GetResponse.ResponseStatus.OK, mode);
        }

        #endregion
    }
}
