﻿using Restup.Webserver.Attributes;
using Restup.Webserver.Models.Contracts;
using Restup.Webserver.Models.Schemas;
using SmartApp.Contracts.Services;

namespace SmartApp.WebPortal
{
    [RestController(InstanceCreationType.Singleton)]
    public sealed class LightOfficeController
    {
        #region members
        
        private static ILightOfficeService _lightOfficeService = null;

        #endregion

        #region constructor
        
        public LightOfficeController()
        {
            _lightOfficeService = Factory.DependencyFactory.GetInstance<ILightOfficeService>();
        }

        #endregion

        #region ILightOfficeService
        
        [UriFormat("/lightoffice/officestates")]
        public IGetResponse GetOfficeStates()
        {
            var states =_lightOfficeService.GetOfficeStates();
            return new GetResponse(GetResponse.ResponseStatus.OK, states);
        }

        [UriFormat("/lightoffice/balklight1/{state}")]
        public IPutResponse SwitchBalkLight1(bool state)
        {
            _lightOfficeService.SwitchBalkLight1(state);
            return new PutResponse(PutResponse.ResponseStatus.OK);
        }

        [UriFormat("/lightoffice/balklight2/{state}")]
        public IPutResponse SwitchBalkLight2(bool state)
        {
            _lightOfficeService.SwitchBalkLight2(state);
            return new PutResponse(PutResponse.ResponseStatus.OK);
        }

        [UriFormat("/lightoffice/balklight5And6/{state}")]
        public IPutResponse SwitchBalk5And6Light(bool state)
        {
            _lightOfficeService.SwitchBalk5And6Light(state);
            return new PutResponse(PutResponse.ResponseStatus.OK);
        }

        [UriFormat("/lightoffice/balklight7And8/{state}")]
        public IPutResponse SwitchBalk7And8Light(bool state)
        {
            _lightOfficeService.SwitchBalk7And8Light(state);
            return new PutResponse(PutResponse.ResponseStatus.OK);
        }

        [UriFormat("/lightoffice/balklight9/{state}")]
        public IPutResponse SwitchBalk9Light(bool state)
        {
            _lightOfficeService.SwitchBalk9Light(state);
            return new PutResponse(PutResponse.ResponseStatus.OK);
        }

        [UriFormat("/lightoffice/balklight10/{state}")]
        public IPutResponse SwitchBalk10Light(bool state)
        {
            _lightOfficeService.SwitchBalk10Light(state);
            return new PutResponse(PutResponse.ResponseStatus.OK);
        }

        [UriFormat("/lightoffice/balklightupper/{state}")]
        public IPutResponse SwitchBalkLightUpper(bool state)
        {
            _lightOfficeService.SwitchBalkLightUpper(state);
            return new PutResponse(PutResponse.ResponseStatus.OK);
        }

        [UriFormat("/lightoffice/mediaboardlight/{state}")]
        public IPutResponse SwitchMediaboardLight(bool state)
        {
            _lightOfficeService.SwitchMediaboardLight(state);
            return new PutResponse(PutResponse.ResponseStatus.OK);
        }

        [UriFormat("/lightoffice/cupboardlight/{state}")]
        public IPutResponse SwitchCupboardLight(bool state)
        {
            _lightOfficeService.SwitchCupboardLight(state);
            return new PutResponse(PutResponse.ResponseStatus.OK);
        }

        #endregion
    }
}
