﻿// ************************************************************************
// Provides client connector for serverbased HeatingManagementService.
// ************************************************************************

var HeatingManagementService = new Object(); // namespace object

// HeatingMangementController/getheatingparameter
HeatingManagementService.GetHeatingParameter = function () {
    var defer = $.Deferred();
    var uri = window.location.protocol + "//" + window.location.host + "/api/v1/heatingmangement/heatingparameter";

    $.getJSON(uri)
        .done(function (data) {
            var params = new Array(); // null, if we get no data, otherwise it will be filled
            if (data) {
                for (var i = 0; i < data.length; i++) {

                    var id = "";
                    var targetTemperature = 0.0;
                    var variance = 0.0;
                    var startHour = 1;
                    var stoppHour = 1;

                    if (data[i].hasOwnProperty("Id")) {
                        id = data[i].Id;
                    }

                    if (data[i].hasOwnProperty("TargetTemperature")) {
                        targetTemperature = data[i].TargetTemperature;
                    }

                    if (data[i].hasOwnProperty("Variance")) {
                        variance = data[i].Variance;
                    }

                    if (data[i].hasOwnProperty("StartHour")) {
                        startHour = data[i].StartHour;
                    }

                    if (data[i].hasOwnProperty("StoppHour")) {
                        stoppHour = data[i].StoppHour;
                    }

                    var heatingParameter = ParameterHeatingModel.ParameterHeating(id, targetTemperature, variance, startHour, stoppHour);
                    params.push(heatingParameter);
                }
            }

            defer.resolve(params);
        })
    .fail(function (ex) {
        defer.fail("Error occured during getting heating parameter from webservice, details " + ex);
    });

    return defer.promise();
};

// HeatingMangementController/setheatingparameter/{parameter}
HeatingManagementService.SetHeatingParameter = function (parameter) {
    var defer = $.Deferred();
    var uri = window.location.protocol + "//" + window.location.host + "/api/v1/heatingmangement/heatingparameter";
    
    $.ajax({
        url: uri,
        type: "PUT",
        data: { "Id": parameter.Id, "TargetTemperature": parameter.TargetTemperature, "Variance": parameter.Variance, "StartHour": parameter.StartHour, "StoppHour": parameter.StoppHour },
        success: function () {
            defer.resolve();
        },
        fail: function (ex) {
            defer.fail("error occured during setting heating parameter with webservice, details " + ex);
        }
    });
    return defer.promise();
};

// HeatingMangementController/deleteheatingparameter/{parameterHeadingId}
HeatingManagementService.DeleteHeatingParameter = function (parameterHeadingId) {
    var defer = $.Deferred();
    var uri = window.location.protocol + "//" + window.location.host + "/api/v1/heatingmangement/heatingparameter/" + parameterHeadingId;

    $.ajax({
        url: uri,
        type: "DELETE",
        success: function () {
            defer.resolve();
        },
        fail: function (ex) {
            defer.fail("error occured during delete heating parameter with webservice, details " + ex);
        }
    });
    return defer.promise();
};

// HeatingMangementController/getheatingstatesbath
HeatingManagementService.GetHeatingStatesBath = function () {
    var defer = $.Deferred();
    var uri = window.location.protocol + "//" + window.location.host + "/api/v1/heatingmangement/heatingstatesbath";

    $.getJSON(uri)
        .done(function (data) {

            var circuitValve = false;
            var temperature = 0.0;
            var humidity = 0.0;

            if (data.hasOwnProperty("CircuitValve")) {
                circuitValve = data.CircuitValve;
            }

            if (data.hasOwnProperty("Temperature")) {
                temperature = data.Temperature;
            }

            if (data.hasOwnProperty("Humidity")) {
                humidity = data.Humidity;
            }

            var bathroomstates = HeatingRoomModel.HeatingRoom(circuitValve, temperature, humidity);
            defer.resolve(bathroomstates);
        })
    .fail(function (ex) {
        defer.fail("Error occured during getting heating states for bath room from webservice, details " + ex);
    });

    return defer.promise();
};

// HeatingMangementController/getheatingstatesoffice
HeatingManagementService.GetHeatingStatesOffice = function () {
    var defer = $.Deferred();
    var uri = window.location.protocol + "//" + window.location.host + "/api/v1/heatingmangement/heatingstatesoffice";

    $.getJSON(uri)
        .done(function (data) {

            var circuitValve = false;
            var temperature = 0.0;
            var humidity = 0.0;

            if (data.hasOwnProperty("CircuitValve")) {
                circuitValve = data.CircuitValve;
            }

            if (data.hasOwnProperty("Temperature")) {
                temperature = data.Temperature;
            }

            if (data.hasOwnProperty("Humidity")) {
                humidity = data.Humidity;
            }

            var officeroomstates = HeatingRoomModel.HeatingRoom(circuitValve, temperature, humidity);
            defer.resolve(officeroomstates);
        })
    .fail(function (ex) {
        defer.fail("Error occured during getting heating states for office room from webservice, details " + ex);
    });

    return defer.promise();
};

// HeatingMangementController/getheatingstatescentral
HeatingManagementService.GetHeatingStatesCentral = function () {
    var defer = $.Deferred();
    var uri = window.location.protocol + "//" + window.location.host + "/api/v1/heatingmangement/heatingstatescentral";

    $.getJSON(uri)
        .done(function (data) {

            var centralPump = false;
           
            if (data.hasOwnProperty("CentralPump")) {
                centralPump = data.CentralPump;
            }

            var centralstates = HeatingCentralModel.HeatingCentral(centralPump);
            defer.resolve(centralstates);
        })
    .fail(function (ex) {
        defer.fail("Error occured during getting heating central states from webservice, details " + ex);
    });

    return defer.promise();
};

// HeatingMangementController/setcentralpump/{state}
HeatingManagementService.SetCentralPump = function (state) {
    var defer = $.Deferred();
    var uri = window.location.protocol + "//" + window.location.host + "/api/v1/heatingmangement/centralpump/" + state;

    $.ajax({
        url: uri,
        type: "PUT",
        success: function () {
            defer.resolve();
        },
        fail: function (ex) {
            defer.fail("error occured during setting central pump with webservice, details " + ex);
        }
    });
    return defer.promise();
};

// HeatingMangementController/setheatingcircuitvalvebath/{state}
HeatingManagementService.SetHeatingCircuitValveBath = function (state) {
    var defer = $.Deferred();
    var uri = window.location.protocol + "//" + window.location.host + "/api/v1/heatingmangement/heatingcircuitvalvebath/" + state;

    $.ajax({
        url: uri,
        type: "PUT",
        success: function () {
            defer.resolve();
        },
        fail: function (ex) {
            defer.fail("error occured during setting bath heating circuit valve with webservice, details " + ex);
        }
    });
    return defer.promise();
};

// HeatingMangementController/setheatingcircuitvalveoffice/{state}
HeatingManagementService.SetHeatingCircuitValveOffice = function (state) {
    var defer = $.Deferred();
    var uri = window.location.protocol + "//" + window.location.host + "/api/v1/heatingmangement/heatingcircuitvalveoffice/" + state;

    $.ajax({
        url: uri,
        type: "PUT",
        success: function () {
            defer.resolve();
        },
        fail: function (ex) {
            defer.fail("error occured during setting office heating circuit valve with webservice, details " + ex);
        }
    });
    return defer.promise();
};

// HeatingMangementController/setservicemode/{servicemode}
HeatingManagementService.SetServiceMode = function (servicemode) {
    var defer = $.Deferred();
    var uri = window.location.protocol + "//" + window.location.host + "/api/v1/heatingmangement/servicemode/" + servicemode ;

    $.ajax({
        url: uri,
        type: "PUT",
        success: function () {
            defer.resolve();
        },
        fail: function (ex) {
            defer.fail("error occured during setting heating service mode with webservice, details " + ex);
        }
    });
    return defer.promise();
};

// HeatingMangementController/getservicemode
HeatingManagementService.GetServiceMode = function () {
    var defer = $.Deferred();
    var uri = window.location.protocol + "//" + window.location.host + "/api/v1/heatingmangement/servicemode";

    $.getJSON(uri)
        .done(function (data) {
            // I expect return value (false/true)
            var serviceMode = data;
            
            defer.resolve(serviceMode);
        })
    .fail(function (ex) {
        defer.fail("Error occured during getting heating service mode from webservice, details " + ex);
    });

    return defer.promise();
};